#include "../../include/types.h"
#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/util.h"

#include <stdlib.h>
#include <string.h>

bool lm_pool_path_set_dir(lm_pool_t *pool, char *dir){
  free(pool->paths.dir);
  pool->paths.dir = NULL;

  if(NULL == dir)
    return true;

  if(exists(dir) && (is_file(dir) || !can_read(dir) || !can_write(dir))){
    lm_error_set(LM_ERR_PoolBadDir);
    return false;
  }

  pool->paths.dir = strdup(dir);
  pool->paths.info_file = join_alloc(dir, "INFO"); 
  pool->paths.list_file = join_alloc(dir, "LIST"); 
  pool->paths.list_dir  = join_alloc(dir, "LIST_extracted"); 
  
  if(exists(pool->paths.info_file) && (!is_file(pool->paths.info_file) || !can_read(pool->paths.info_file) || !can_write(pool->paths.info_file))){
    lm_pool_path_free(pool);
    lm_error_set(LM_ERR_PoolBadPaths);
    return false;
  }
  
  if(exists(pool->paths.list_file) && (!is_file(pool->paths.list_file) || !can_read(pool->paths.list_file) || !can_write(pool->paths.list_file))){
    lm_pool_path_free(pool);
    lm_error_set(LM_ERR_PoolBadPaths);
    return false;
  }

  if(exists(pool->paths.list_dir) && (is_file(pool->paths.list_dir) || !can_read(pool->paths.list_dir) || !can_write(pool->paths.list_dir))){
    lm_pool_path_free(pool);
    lm_error_set(LM_ERR_PoolBadPaths);
    return false;
  }
 
  return true;
}

bool lm_pool_path_is_empty(lm_pool_t *pool){
  return NULL == pool->paths.info_file ||
         NULL == pool->paths.list_file ||
         NULL == pool->paths.list_dir  ||
         NULL == pool->paths.dir;
}

void lm_pool_path_free(lm_pool_t *pool){
  free(pool->paths.dir);
  free(pool->paths.list_dir);
  free(pool->paths.info_file);
  free(pool->paths.list_file);
  bzero(&pool->paths, sizeof(pool->paths));
}
