#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/util.h"
#include "../../include/pool.h"

#include <stdlib.h>
#include <string.h>

lm_pool_t *lm_pool_new(char *name, char *url) {
  lm_pool_t *pool = malloc(sizeof(lm_pool_t));
  bzero(pool, sizeof(lm_pool_t));
  
  pool->available = true;
  pool->loaded    = false;
  pool->name      = name;

  if (NULL != url && !lm_url_init(&pool->url, url)) {
    free(pool);
    return NULL;
  }

  if (!eq(pool->url.protocol, "mptp")) {
    lm_error_set(LM_ERR_PoolNoSupport);
    lm_pool_free(pool);
    return NULL;
  }

  return pool;
}

void lm_pool_test(lm_pool_t *pool) {
  lm_mptp_t packet;
  lm_mptp_init(&packet, true, MPTP_C2S_PING, true);

  lm_mptp_set_host(&packet, pool->url.host);
  lm_mptp_set_data(&packet, pool->url.path, strlen(pool->url.path));

  int sock = lm_mptp_client_connect(pool->url.host, pool->url.port);
  if (sock == -1) {
    pool->available = false;
    return;
  }

  if (!lm_mptp_client_send(sock, &packet)) {
    pool->available = false;
    goto end;
  }

  if (!lm_mptp_client_recv(sock, &packet)) {
    pool->available = false;
    goto end;
  }

  if (!lm_mptp_client_verify(&packet)) {
    pool->available = false;
    goto end;
  }

  pool->available = MPTP_FLAGS_CODE(&packet) == MPTP_S2C_PONG;
  if(!pool->available)
    lm_error_set(LM_ERR_PoolTestNotPong);
end:
  lm_mptp_close(sock);
  return;
}

void lm_pool_free(lm_pool_t *pool) {
  lm_url_free(&pool->url);
  lm_pool_info_free(pool);
  lm_pool_list_free(pool);
  free(pool);
}
